import random

import cocos
from cocos.director import director
from cocos import sprite, path, euclid
from cocos.actions import Bezier, FadeTo, RotateBy, Delay
import pyglet
import math
import os
import time

pyglet.resource.path.append("/home/facundo/Escritorio/present_cierre/fotosreducidas")
pyglet.resource.reindex()

def _(n):
    return n # * 10

PANTALLA = dict(width=1024, height=768)
CENTRO = PANTALLA["width"]//2, PANTALLA["height"]//2

def direct_bezier(p0, p1, p2, p3):
    '''Given four points, returns a bezier path that go through them.

    It starts in p0, finish in p3, and pass through p1 and p2 in t=0.4 and
    t=0.6 respectively.
    '''

    def _one_dim(p0xy, B1xy, B2xy, p3xy):
        '''Calculates the p1 and p2 to draw through B1 and B2 given p0 and p3.

        p0: P sub 0 of bezier, it's also B(0)
        B1: B(0.4)
        B2: B(0.6)
        p3: P sub 3 of bezier, it's also B(1)
        '''
        p2xy = (1.5 * B2xy - B1xy + 0.12 * p0xy - 0.26 * p3xy) / 0.36
        p1xy = (B2xy - 0.064 * p0xy - 0.432 * p2xy - 0.216 * p3xy) / 0.288
        return p1xy, p2xy

    bp1x, bp2x = _one_dim(p0[0], p1[0], p2[0], p3[0])
    bp1y, bp2y = _one_dim(p0[1], p1[1], p2[1], p3[1])

    bp1 = bp1x, bp1y
    bp2 = bp2x, bp2y

    bezier_path = path.Bezier(p0, p3, bp1, bp2)
    return bezier_path


class Recorder(object):
    def __init__(self):
        self.c = 0
        buffers = pyglet.image.get_buffer_manager()
        self.cbuf = buffers.get_color_buffer()
    def __call__(self, dt):
        n = "filminas/img%06d.png" % self.c
        self.c += 1
        self.cbuf.save(n)


def get_random_afuera(imagw, imagh):
    himagw = imagw // 2
    himagh = imagh // 2
    while True:
        x = random.randint(-imagw, PANTALLA["width"] + imagw)
        y = random.randint(-imagh, PANTALLA["height"] + imagh)
        if -himagw < x < PANTALLA["width"] + himagw:
            continue
        if -himagh < y < PANTALLA["height"] + himagh:
            continue
        break
    return euclid.Point2(x, y)


def get_random_centro():
    hip = random.randint(30, 100)
    ang = random.random() * 2 * math.pi
    x = PANTALLA["width"] // 2 + hip * math.cos(ang)
    y = PANTALLA["height"] // 2 + hip * math.sin(ang)
    return euclid.Point2(x, y)


def get_foto():
    imags = os.listdir("/home/facundo/Escritorio/present_cierre/fotosreducidas")
    imags.sort()
    for imag in imags[:3]:
        yield imag


class TestLayer(cocos.layer.Layer):
    def __init__(self):
        super( TestLayer, self ).__init__()

        self.imags = get_foto()
        self.curve_start = None
        self.fly_image()
        self.schedule_interval(self.fly_image, 4)

#        r = Recorder()
#        self.schedule_interval(r, _(1/30.0))

    # mencoder mf://*.png -mf fps=30 -o foo.avi -ovc lavc

    def fly_image(self, dt=None):
        '''Makes image fly.'''
        # get the image
        try:
            imag = self.imags.next()
        except StopIteration:
            self.unschedule(self.fly_image)
            return
        sp = sprite.Sprite(imag)

        # points, relative to window
        if self.curve_start is None:
            self.curve_start = get_random_afuera(sp.width, sp.height)
        curve_exit = get_random_afuera(sp.width, sp.height)
        c1 = get_random_centro()
        c2 = get_random_centro()
        rot = random.randint(5,30) * (-1) ** random.randint(0,1)

        # load the sprite
        sp.position = self.curve_start
        sp.opacity = 0
        sp.rotation = rot
        self.add(sp)

        # make the points relative to image
        c1 -= self.curve_start
        c2 -= self.curve_start
        rel_exit = curve_exit - self.curve_start

        # make it fly
        bez = direct_bezier((0,0), c1, c2, rel_exit)
        sp.do(Bezier(bez, _(6)) |
              FadeTo(255, _(2)) + Delay(_(2)) + FadeTo(0, _(2)) |
              RotateBy(-rot*2, _(6))
             )

        # store in curve_start the oposite of this exit
        x, y = curve_exit
        x = PANTALLA["width"] - x
        y = PANTALLA["height"] - y
        self.curve_start = x, y


if __name__ == "__main__":
    director.init(**PANTALLA)
    test_layer = TestLayer ()
    main_scene = cocos.scene.Scene (test_layer)
#    director.show_FPS = True
    director.run (main_scene)
